@@ -1,5 +1,4 @@
-import os
-import sys
+import os, sys
 
 # All modules to test.
 MODULES = (
@@ -21,10 +20,11 @@ MODULES = (
     "nstd_os nstd_os_def",
     "nstd_os nstd_os_io",
     "nstd_proc",
+    "nstd_rand",
     "nstd_string",
-    "nstd_sys",
     "nstd_thread",
-    "nstd_time"
+    "nstd_time",
+    "nstd_vec"
 )
 
 # Iterates through each module, running cargo check for each of them.
