@@ -1,7 +1,8 @@
+//! UDP networking.
 use crate::{
-    collections::vec::NSTDVec,
     core::{def::NSTDErrorCode, slice::NSTDSlice, str::NSTDStr},
     string::NSTDString,
+    vec::NSTDVec,
 };
 use std::net::UdpSocket;
 
@@ -10,9 +11,14 @@ pub type NSTDUDPSocket = *mut UdpSocket;
 
 /// Creates a UDP socket bound to the given address. Call `nstd_net_udp_socket_close` to free
 /// memory allocated by this function and close the socket.
-/// Parameters:
-///     `const NSTDStr *const addr` - The address to listen on, formatted as "IP:Port".
-/// Returns: `NSTDUDPSocket socket` - The UDP socket, null on error.
+///
+/// # Parameters
+///
+/// - `const NSTDStr *const addr` - The address to listen on, formatted as "IP:Port".
+///
+/// # Returns
+///
+/// `NSTDUDPSocket socket` - The UDP socket, null on error.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_bind(addr: &NSTDStr) -> NSTDUDPSocket {
     if let Ok(addr) = std::str::from_utf8(addr.bytes.as_byte_slice()) {
@@ -24,10 +30,16 @@ pub unsafe extern "C" fn nstd_net_udp_socket_bind(addr: &NSTDStr) -> NSTDUDPSock
 }
 
 /// Connects a UDP socket to a remote address.
-/// Parameters:
-///     `const NSTDUDPSocket socket` - The socket to connect.
-///     `const NSTDStr *const addr` - The remote address to connect to.
-/// Returns: `NSTDErrorCode errc` - Nonzero on error.
+///
+/// # Parameters
+///
+/// - `const NSTDUDPSocket socket` - The socket to connect.
+///
+/// - `const NSTDStr *const addr` - The remote address to connect to.
+///
+/// # Returns
+///
+/// `NSTDErrorCode errc` - Nonzero on error.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_connect(
     socket: NSTDUDPSocket,
@@ -42,10 +54,16 @@ pub unsafe extern "C" fn nstd_net_udp_socket_connect(
 }
 
 /// Receives bytes sent from the connected address.
-/// Parameters:
-///     `const NSTDUDPSocket socket` - The socket to receive bytes on.
-///     `const NSTDUSize num` - Number of bytes to receive.
-/// Returns: `NSTDVec bytes` - The bytes received.
+///
+/// # Parameters
+///
+/// - `const NSTDUDPSocket socket` - The socket to receive bytes on.
+///
+/// - `const NSTDUSize num` - Number of bytes to receive.
+///
+/// # Returns
+///
+/// `NSTDVec bytes` - The bytes received.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_receive(socket: NSTDUDPSocket, num: usize) -> NSTDVec {
@@ -58,12 +76,22 @@ pub unsafe extern "C" fn nstd_net_udp_socket_receive(socket: NSTDUDPSocket, num:
 }
 
 /// Receives bytes sent to a UDP socket.
-/// NOTE: This creates a new `NSTDString` so make sure `ip` is freed before using this function.
-/// Parameters:
-///     `const NSTDUDPSocket socket` - The socket to receive bytes from.
-///     `const NSTDUSize num` - Number of bytes to receive.
-///     `NSTDString *const ip` - Returns as the socket IP address the bytes came from.
-/// Returns: `NSTDVec bytes` - The bytes received.
+///
+/// # Note
+///
+/// This creates a new `NSTDString` so make sure `ip` is freed before using this function.
+///
+/// # Parameters
+///
+/// - `const NSTDUDPSocket socket` - The socket to receive bytes from.
+///
+/// - `const NSTDUSize num` - Number of bytes to receive.
+///
+/// - `NSTDString *const ip` - Returns as the socket IP address the bytes came from.
+///
+/// # Returns
+///
+/// `NSTDVec bytes` - The bytes received.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_receive_from(
     socket: NSTDUDPSocket,
@@ -82,11 +110,18 @@ pub unsafe extern "C" fn nstd_net_udp_socket_receive_from(
 }
 
 /// Sends bytes from a UDP socket to another.
-/// Parameters:
-///     `const NSTDUDPSocket socket` - The UDP socket.
-///     `const NSTDSlice *const bytes` - The bytes to send.
-///     `NSTDUSize *const size` - Returns as number of bytes actually sent.
-/// Returns: `NSTDErrorCode errc` - Nonzero on error.
+///
+/// # Parameters
+///
+/// - `const NSTDUDPSocket socket` - The UDP socket.
+///
+/// - `const NSTDSlice *const bytes` - The bytes to send.
+///
+/// - `NSTDUSize *const size` - Returns as number of bytes actually sent.
+///
+/// # Returns
+///
+/// `NSTDErrorCode errc` - Nonzero on error.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_send(
     socket: NSTDUDPSocket,
@@ -101,12 +136,20 @@ pub unsafe extern "C" fn nstd_net_udp_socket_send(
 }
 
 /// Sends bytes from a UDP socket to another.
-/// Parameters:
-///     `const NSTDUDPSocket socket` - The UDP socket.
-///     `const NSTDStr *const addr` - The address to send the bytes to.
-///     `const NSTDSlice *const bytes` - The bytes to send.
-///     `NSTDUSize *const size` - Returns as number of bytes actually sent.
-/// Returns: `NSTDErrorCode errc` - Nonzero on error.
+///
+/// # Parameters
+///
+/// - `const NSTDUDPSocket socket` - The UDP socket.
+///
+/// - `const NSTDStr *const addr` - The address to send the bytes to.
+///
+/// - `const NSTDSlice *const bytes` - The bytes to send.
+///
+/// - `NSTDUSize *const size` - Returns as number of bytes actually sent.
+///
+/// # Returns
+///
+/// `NSTDErrorCode errc` - Nonzero on error.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_send_to(
     socket: NSTDUDPSocket,
@@ -124,8 +167,10 @@ pub unsafe extern "C" fn nstd_net_udp_socket_send_to(
 }
 
 /// Closes and frees memory of a UDP socket.
-/// Parameters:
-///     `NSTDUDPSocket *const socket` - Pointer to the UDP socket.
+///
+/// # Parameters
+///
+/// - `NSTDUDPSocket *const socket` - Pointer to the UDP socket.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_net_udp_socket_close(socket: *mut NSTDUDPSocket) {
@@ -137,5 +182,5 @@ pub unsafe extern "C" fn nstd_net_udp_socket_close(socket: *mut NSTDUDPSocket) {
 #[inline]
 unsafe fn null_vec() -> NSTDVec {
     let null_slice = crate::core::slice::nstd_core_slice_new(0, 0, std::ptr::null_mut());
-    crate::collections::vec::nstd_collections_vec_from_existing(0, &null_slice)
+    crate::vec::nstd_vec_from_existing(0, &null_slice)
 }
